Package: sgo
Title: Simple Geographical Operations (with OSGB36)
Version: 0.9.2
Authors@R: person("Carlos", "Lozano Ruiz", 
  email = "carloslozanoruiz@outlook.com", role = c("aut", "cre"))
URL: https://github.com/clozanoruiz/sgo
BugReports: https://github.com/clozanoruiz/sgo/issues
Language: en-GB
Description: Methods focused in performing the OSGB36/ETRS89 transformation 
  (Great Britain and the Isle of Man only) by using the Ordnance Survey's 
  OSTN15/OSGM15 transformation model. Calculation of distances and areas from 
  sets of points defined in any of the supported Coordinated Systems is also
  available.
Depends: R (>= 3.5.0)
Suggests: maps, tinytest
License: BSD_2_clause + file LICENSE
Copyright: OSTN transformation data included in this module remains
        Crown Copyright (C) 2016 under the terms of the BSD licence:
        (c) Copyright and database rights Ordnance Survey Limited 2016,
        (c) Crown copyright and database rights Land & Property
        Services 2016 and/or (c) Ordnance Survey Ireland, 2016. All
        rights reserved.
Encoding: UTF-8
ByteCompile: yes
RoxygenNote: 7.2.1
Collate: 'sgo.R' 'sgo_area.R' 'sgo_bng.R' 'sgo_distance.R' 'sgo_laea.R'
        'sgo_ngr.R' 'sgo_points.R' 'sgo_set_gcs.R' 'sgo_wgs84.R'
        'sgo_transform.R'
NeedsCompilation: no
Packaged: 2022-09-23 15:55:32 UTC; Carlos
Author: Carlos Lozano Ruiz [aut, cre]
Maintainer: Carlos Lozano Ruiz <carloslozanoruiz@outlook.com>
Repository: CRAN
Date/Publication: 2022-09-23 16:20:02 UTC
Built: R 4.6.0; ; 2026-04-27 03:11:00 UTC; unix
