# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,sgo_points)
S3method(as.list,sgo_points)
S3method(print,sgo_points)
S3method(sgo_area,sgo_points)
S3method(sgo_bng_lonlat,sgo_points)
S3method(sgo_bng_ngr,sgo_points)
S3method(sgo_cart_lonlat,sgo_points)
S3method(sgo_coordinates,sgo_points)
S3method(sgo_distance,sgo_points)
S3method(sgo_en_wgs84,sgo_points)
S3method(sgo_etrs_laea,sgo_points)
S3method(sgo_laea_etrs,sgo_points)
S3method(sgo_lonlat_bng,sgo_points)
S3method(sgo_lonlat_cart,sgo_points)
S3method(sgo_ngr_bng,data.frame)
S3method(sgo_ngr_bng,default)
S3method(sgo_ngr_bng,list)
S3method(sgo_points,data.frame)
S3method(sgo_points,list)
S3method(sgo_points,matrix)
S3method(sgo_set_gcs,sgo_points)
S3method(sgo_transform,sgo_points)
S3method(sgo_wgs84_en,sgo_points)
export(sgo_area)
export(sgo_bng_lonlat)
export(sgo_bng_ngr)
export(sgo_cart_lonlat)
export(sgo_coordinates)
export(sgo_distance)
export(sgo_en_wgs84)
export(sgo_etrs_laea)
export(sgo_laea_etrs)
export(sgo_lonlat_bng)
export(sgo_lonlat_cart)
export(sgo_ngr_bng)
export(sgo_points)
export(sgo_set_gcs)
export(sgo_transform)
export(sgo_wgs84_en)
